/*
 * Copyright (c) 2015-2021 by inovat
 * innovative systeme - verkehr - tunnel - technik,
 * Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.bsvrz.kex.tls.osi7.AdditionalCommunicationPartner
 *
 * de.bsvrz.kex.tls.osi7.AdditionalCommunicationPartner is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * de.bsvrz.kex.tls.osi7.AdditionalCommunicationPartner is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.kex.tls.osi7.AdditionalCommunicationPartner.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Diese Datei ist Teil von de.bsvrz.kex.tls.osi7.AdditionalCommunicationPartner.
 *
 * de.bsvrz.kex.tls.osi7.AdditionalCommunicationPartner ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 *
 * de.bsvrz.kex.tls.osi7.AdditionalCommunicationPartner wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 *
 * Sie sollten eine Kopie der GNU General Public License zusammen mit diesem
 * Programm erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * An der Krautwiese 37
 * D-53783 Eitorf
 * +49 (0)2243 8464 193
 * info@inovat.de
 * www.inovat.de
 */

package de.bsvrz.kex.tls.osi7;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.*;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Klasse zur Verwaltung der AdditionalCommunicationPartner (TLS ähnliche Kommunikationspartner wie z.B. WanCom).
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Dipl.-Ing. Hans Christian Kniß (HCK)
 */

public class AdditionalCommunicationPartner {
    // ------------------------------------------------------------------------
    // VARIABLENDEKLARATIONEN
    // ------------------------------------------------------------------------

    // Sonstige ---------------------------------------------------------------
    /** DebugLogger für Debug-Ausgaben */
    private static final Debug debug = Debug.getLogger();
    // Private ----------------------------------------------------------------
    /** DaV Interface */
    private final ClientDavInterface _daf;

    /** Zielobjekt ("virtuelle" DE im Sinne der TLS), zu dem die Daten gehören. */
    private final ConfigurationObject _referencedObject;

    /** Sender (typ.gerät), der die Daten sendet. */
    private final ConfigurationObject _sender;

    /** Name des UmsetzungsModul für die Konvertierung der Telegramme in Attributgruppen (und zurück) */
    private final String _conversionModulName;
    /** De-Kanal-Nummer des DE-Blöcks. */
    private final int _deChannel;
    /** Funktionsgruppe des DE. */
    private final int _fg;
    /** OSI7 KnotenNummer. */
    private final int _osi7Adress;
    /** Senderadresse des Gerätes, dassdie Daten sendet. */
    private final int _senderOsi7Adress;
    /** Markiert den Eintrag als bereits abgearbeitet (als gelöscht markiert). */
    private boolean _isDeleted = false;

    // ------------------------------------------------------------------------
    // KONSTRUKTOREN  (und vom Konstruktor verwende Methoden)
    // ------------------------------------------------------------------------

    /**
     * Erzeugt ein Objekt vom Typ AdditionalCommunicationPartner
     *
     * @param daf  DaV.
     * @param data Daten.
     */
    public AdditionalCommunicationPartner(ClientDavInterface daf, Data data) {
        _daf = daf;
        _referencedObject = (ConfigurationObject) data.getItem("Pid").asReferenceValue().getSystemObject();
        _osi7Adress = data.getItem("KnotenNummer").asUnscaledValue().intValue();
        _fg = data.getItem("FG").asUnscaledValue().intValue();
        _deChannel = data.getItem("DEKanal").asUnscaledValue().intValue();
        _conversionModulName = data.getItem("UmsetzungsModul").asTextValue().getText();
        _sender = (ConfigurationObject) data.getItem("PidSender").asReferenceValue().getSystemObject();

        DataModel config = _daf.getDataModel();
        AttributeGroup devicePropertiesAtg = config.getAttributeGroup("atg.gerät");
        Data deviceProperties = _sender.getConfigurationData(devicePropertiesAtg);

        _senderOsi7Adress = deviceProperties.getScaledValue("KnotenNummer").intValue();
    }

    // ------------------------------------------------------------------------
    // GETTER / SETTER METHODEN
    // ------------------------------------------------------------------------

    public String getConversionModulName() {
        return _conversionModulName;
    }

    public int getDeChannel() {
        return _deChannel;
    }

    public int getFg() {
        return _fg;
    }

    /**
     * Liefert einen Hashkey auf Basis der KnotenNummer als String in Dezimaldarstellung (0..16777215).
     *
     * @return Hashkey auf Basis der KnotenNummer
     */
    public String getKey() {
        return (String.valueOf(_osi7Adress));
    }

    public int getOsi7Adress() {
        return _osi7Adress;
    }

    public ConfigurationObject getReferencedObject() {
        return _referencedObject;
    }

    public ConfigurationObject getSender() {
        return _sender;
    }

    public int getSenderOsi7Adress() {
        return _senderOsi7Adress;
    }

    public boolean isDeleted() {
        return _isDeleted;
    }

    public void setDeleted(boolean deleted) {
        _isDeleted = deleted;
    }

    // ------------------------------------------------------------------------
    // KANONISCHE METHODEN
    // ------------------------------------------------------------------------

    public String toString() {
        return "AdditionalCommunicationPartner{" + "_referencedObject=" + _referencedObject + ", _osi7Adress=" + _osi7Adress + ", _fg=" + _fg +
               ", _deChannel=" + _deChannel + ", _conversionModulName='" + _conversionModulName + "'" + ", _sender=" + _sender +
               ", _senderOsi7Adress=" + _senderOsi7Adress + ", _isDeleted=" + _isDeleted + "}";
    }

    // ------------------------------------------------------------------------
    // SONSTIGE METHODEN
    // ------------------------------------------------------------------------

    /**
     * Liefert eine Hashkey als String auf Basis "SenderOsi7Adresse-Osi7Adresse-DeNummer-Fg".
     *
     * @return Hashkey als String auf Basis "SenderOsi7Adresse-Osi7Adresse-DeNummer-Fg".
     */
    public String getSenderReceiverKey() {
        String key = new StringBuffer().append(String.valueOf(_senderOsi7Adress)).append("-").append(String.valueOf(_osi7Adress)).append("-")
            .append(String.valueOf(_deChannel)).append("-").append(String.valueOf(_fg)).toString();
        return key;
    }
}
